@@ -4,6 +4,7 @@ import warnings
 from django.db import models
 from django.conf import settings
 from django.contrib.postgres import fields as postgres_fields
+from django.core.exceptions import ImproperlyConfigured
 
 from sqlalchemy import types as default_types
 from sqlalchemy.dialects import (
@@ -274,24 +275,21 @@ mapping = {
         '__oracle_type__': oracle_types.NUMBER,
         '__oracle_type_kwargs__': {'precision': 1},
     },
-    models.NullBooleanField: {
-        '__default_type__': default_types.BOOLEAN,
-        '__postgresql_type__': postgresql_types.BOOLEAN,
-        '__mysql_type__': mysql_types.BOOLEAN,
-        '__oracle_type__': oracle_types.NUMBER,
-        '__oracle_type_kwargs__': {'precision': 1},
-        'nullable': True,
-    },
     models.ForeignKey: {
         '__callback__': lambda f: {
             '__callback__': lambda f: (mapping[type(f.target_field)], f.target_field),
             '__rel_kwargs__': {
                 '__logical_name__': f.name,
-                '__back__': f.related_query_name(),
-                '__target__': f.related_model()._meta.db_table,
+                '__back__': f.related_query_name().rstrip('+').lower(),
+                '__back_target__': f.model._meta.db_table,
+                '__target__': f.related_model._meta.db_table,
+                '__model__': f.model,
+                '__related_model__': f.related_model,
+                'foreign_keys': f"[{f.model._meta.db_table}.c.{f.column}]",
+                'remote_side': f"[{f.model._meta.db_table}.c.{f.target_field.column}]" if f.model == f.related_model else None,
             },
             '__fk_kwargs__': {
-                'column': '{meta.db_table}.{meta.pk.attname}'.format(meta=f.related_model._meta),
+                'column': '{meta.db_table}.{meta.pk.column}'.format(meta=f.related_model._meta),
                 'ondelete': f.remote_field.on_delete.__name__,
             },
         },
@@ -301,12 +299,17 @@ mapping = {
             '__callback__': lambda f: (mapping[type(f.target_field)], f.target_field),
             '__rel_kwargs__': {
                 '__logical_name__': f.name,
-                '__back__': f.related_query_name(),
-                '__target__': f.related_model()._meta.db_table,
+                '__back__': f.related_query_name().rstrip('+').lower(),
+                '__back_target__': f.model._meta.db_table,
+                '__target__': f.related_model._meta.db_table,
+                '__model__': f.model,
+                '__related_model__': f.related_model,
+                'foreign_keys': f"[{f.model._meta.db_table}.c.{f.column}]",
+                'remote_side': f"[{f.model._meta.db_table}.c.{f.target_field.column}]" if f.model == f.related_model else None,
                 'uselist': False,
             },
             '__fk_kwargs__': {
-                'column': '{meta.db_table}.{meta.pk.attname}'.format(meta=f.related_model._meta),
+                'column': '{meta.db_table}.{meta.pk.column}'.format(meta=f.related_model._meta),
                 'ondelete': f.remote_field.on_delete.__name__,
             },
         },
@@ -318,13 +321,36 @@ mapping = {
                 '__target_field__': f.field.m2m_target_field_name(),
                 '__remote_primary_field__': f.field.m2m_column_name(),
                 '__remote_secondary_field__': f.field.m2m_reverse_name(),
-                '__back__': f.field.related_query_name(),
+                '__back__': f.field.related_query_name().rstrip('+').lower(),
+                '__back_target__': f.rel.related_model._meta.db_table,
                 '__target__': f.rel.model._meta.db_table,
+                '__model__': f.rel.related_model,
+                '__related_model__': f.rel.model,
+                'secondary': f.rel.through._meta.db_table,
+                'foreign_keys': (
+                    f"[{f.rel.through._meta.db_table}.c.{f.rel.field._m2m_column_cache}]"
+                    if f.field.model == f.field.related_model else
+                    f"[{f.rel.through._meta.db_table}.c.{f.rel.field._m2m_column_cache}, {f.rel.through._meta.db_table}.c.{f.rel.field._m2m_reverse_column_cache}]"
+                ),
+                'remote_side': f"[{f.field.model._meta.db_table}.c.{f.rel.target_field.column}]" if f.field.model == f.field.related_model else None,
             },
         } if not f.reverse else {}
     },
 }
 
+try:
+    # Deprecated since version 3.1
+    mapping[models.NullBooleanField] = {
+        '__default_type__': default_types.BOOLEAN,
+        '__postgresql_type__': postgresql_types.BOOLEAN,
+        '__mysql_type__': mysql_types.BOOLEAN,
+        '__oracle_type__': oracle_types.NUMBER,
+        '__oracle_type_kwargs__': {'precision': 1},
+        'nullable': True,
+    }
+except AttributeError:
+    pass
+
 try:
     # deprecated
     mapping[models.CommaSeparatedIntegerField] = {
@@ -342,7 +368,6 @@ try:
 except AttributeError:
     pass
 
-
 try:
     mapping[models.PositiveBigIntegerField] = {
         '__default_type__': default_types.BIGINT,
@@ -382,14 +407,21 @@ except AttributeError:
     pass
 
 try:
+    def _parse_array(field, db_type):
+        dimensions = 1
+        while isinstance(field, postgres_fields.ArrayField):
+            field = field.base_field
+            dimensions += 1
+        return {"item_type": mapping[type(field)].get(f"__{db_type}_type__"), "dimensions": dimensions}
+
     mapping[postgres_fields.ArrayField] = {
         '__default_type__': postgresql_types.ARRAY,
         '__postgresql_type__': postgresql_types.ARRAY,
         '__mysql_type__': default_types.ARRAY,
         '__oracle_type__': default_types.ARRAY,
         '__callback__': lambda f: {
-            '__default_type_kwargs__': {'item_type': mapping[type(f.base_field)]['__default_type__']},
-            '__postgresql_type_kwargs__': {'item_type': mapping[type(f.base_field)].get('__postgresql_type__') or mapping[type(f.base_field)]['__default_type__']},
+            '__default_type_kwargs__': _parse_array(f.base_field, "default"),
+            '__postgresql_type_kwargs__': _parse_array(f.base_field, "postgresql"),
         },
     }
 except AttributeError:
@@ -414,6 +446,15 @@ JSONType, JSONRule = 'JSONType', {
 
 mapping[JSONType] = JSONRule
 
+try:
+    mapping[models.JSONField] = {
+        **JSONRule,
+        '__postgresql_type__': postgresql_types.JSONB,
+        '__oracle_type__': oracle_types.NCLOB,
+    }
+except AttributeError:
+    pass
+
 try:
     mapping[postgres_fields.JSONField] = {
         **JSONRule,
@@ -500,6 +541,6 @@ try:
     from .geoalchemy2 import geo_mapping
     mapping.update(geo_mapping)
 
-except (ImportError, AttributeError) as e:
+except (ImportError, AttributeError, ImproperlyConfigured) as e:
     if D2A_CONFIG.get('USE_GEOALCHEMY2', False):
         warnings.warn('An error occured: {}. HINT: GeoAlchemy2 should be installed when you use GeoDjango.'.format(e))
